package InternalStructure;

import CommonClasses.CommunicationContainer;
import CommonClasses.CommunicationKind;
import CommonClasses.Move;
import CommonClasses.Player;
import CommonInterfaces.InterfaceProxyPlayer;
import InterfaceParts.PortPlayer;
import InterfaceParts.PortPlayerOutbox;
import InterfaceParts.PortServer;
import InterfaceParts.PortServerOutbox;

public class InternalServerProxy implements InterfaceProxyPlayer {
	
	protected PortServer portServer;
	protected PortPlayer portPlayer;
	
	public void setPortServer (PortServer aPortServer) {
		portServer = aPortServer;
	}
	
	public void setPortPlayer (PortPlayer aPortPlayer) {
		portPlayer = aPortPlayer;
	}

	@Override
	public void confirmConnection() {
		boolean serverConnected = portServer.getConnected();
		if (! serverConnected) {
			PortPlayerOutbox playerOutbox = (PortPlayerOutbox) portPlayer.getOutbox();
			PortServerOutbox serverOutbox = (PortServerOutbox) portServer.getOutbox();
			serverOutbox.disconnect();		//		disconnectNG()
			CommunicationContainer result = new CommunicationContainer();
			result.setKind(CommunicationKind.connectionNotification);
			result.setContent("Server not connected");
			playerOutbox.communicate(result);
		}		
	}

	@Override
	public boolean connect(String arg0) {
		PortServerOutbox serverOutbox = (PortServerOutbox) portServer.getOutbox();
		serverOutbox.connect(arg0);
		serverOutbox.start();
		return portServer.getConnected();
	}

	@Override
	public void disconnect(String arg0) {
		PortServerOutbox serverOutbox = (PortServerOutbox) portServer.getOutbox();
		CommunicationContainer  request = new CommunicationContainer();
		request.setKind(CommunicationKind.disconnectRequest);
		request.setContent(arg0);		
		portServer.setConnected(false);
		serverOutbox.send(request);	
	}

	@Override
	public void getConnectedPlayers() {
		PortServerOutbox serverOutbox = (PortServerOutbox) portServer.getOutbox();
		CommunicationContainer  request = new CommunicationContainer();
		request.setKind(CommunicationKind.playerIdRequest);
		request.setContent("");
		serverOutbox.send(request);	
	}

	@Override
	public void giveUp(String arg0) {
		PortServerOutbox serverOutbox = (PortServerOutbox) portServer.getOutbox();
		CommunicationContainer  request = new CommunicationContainer();
		request.setKind(CommunicationKind.giveUpRequest);
		request.setContent(arg0);
		serverOutbox.send(request);	
	}

	@Override
	public void makeMove(Move arg0) {
		PortServerOutbox serverOutbox = (PortServerOutbox) portServer.getOutbox();
		serverOutbox.send(arg0);	
	}

	@Override
	public void sendMessage(String arg0) {
		PortServerOutbox serverOutbox = (PortServerOutbox) portServer.getOutbox();
		serverOutbox.sendMessage(arg0);	
	}

	@Override
	public void start(Player arg0) {
		PortServerOutbox serverOutbox = (PortServerOutbox) portServer.getOutbox();
		serverOutbox.send(arg0);	
	}

}
