package InternalStructure;

import CommonClasses.Game;
import CommonClasses.Move;
import CommonClasses.Player;
import CommonClasses.State;

// Game.gameCode
// 1 - game not started
// 2 - game finished with quitting
// 3 - irregular move (game in progress)
// 4 - next player (game in progress)
// 5 - game with winner
// 6 - game tied

public class Board {
	
	protected Position positions[][] = new Position[3][3];
	protected Player player1 = null;
	protected Player player2 = null;
	protected boolean finished = false;
	protected boolean winner = false;
	
	public Board() {
		for (int x=0; x<3; x++){
			for (int y=0; y<3; y++){
				positions[x][y] = new Position();		
			};
		};
	}

	public Game setInitialState(Player argPlayer1, Player argPlayer2) {
		player1 = argPlayer1;
		player2 = argPlayer2;
		Game game = new Game(); // gameCode = 1
		game.setPlayer1(player1);
		game.setPlayer2(player2);
		State state = new State();
		Player player = this.getEnabledPlayer();
		String name = player.getName();
		String message = "Player - " + name;
		state.setStateMessage(message);
		game.setGameState(state);		
		return game;
	}

	public Game finishWithdrawal(Game game, Player player) {
		game.setGameCode(2);
		String name = player.getName();
		String message = "Player " + name + " gave up";
		State state = game.getGameState();
		state.setStateMessage(message);
		return game;
	}
	
	public Game treatMove(Game game, Move move) {
		State state = game.getGameState();
		this.setEmpty();
		player1 = game.getPlayer1();
		player2 = game.getPlayer2();		
		for (int x=0; x<3; x++){
			for (int y=0; y<3; y++){
				if (player1.getSymbol() == state.getValue(x+1, y+1)) {
					positions[x][y].setOccupant(player1);
				} else {
					if (player2.getSymbol() == state.getValue(x+1, y+1)) {
						positions[x][y].setOccupant(player2);
					}
				}			
			};
		};		
		Position selectedPosition = this.getPosition(move);
		if (selectedPosition.isOccupied()) {
			game.setGameCode(3);
			state.setStateMessage("Irregular move. Try again.");
		} else {
			Player player = this.getEnabledPlayer();
			selectedPosition.setOccupant(player);
			this.evaluateWinner(move);
			if (! winner) this.evaluateFinished();
			if (winner) {
				game.setGameCode(5);
				player1.setTurn(false);
				player2.setTurn(false);
			} else {
				if (finished) {
					game.setGameCode(6);
					player1.setTurn(false);
					player2.setTurn(false);
				} else {
					game.setGameCode(4);
				}
			}
			this.toogleEnabledPlayer();
			State newState = this.getState();
			game.setGameState(newState);
		}
		return game;
	}

	private Player getEnabledPlayer() {
		if (player1.getTurn()) {
			return player1;
		} else {
			return player2;
		}		
	}
	
	private void toogleEnabledPlayer() {
		if (player1.getTurn()) {
			player1.setTurn(false);
			player2.setTurn(true);
		} else {
			player1.setTurn(true);
			player2.setTurn(false);
		}		
	}
	
	private void setEmpty() {
		for (int x=0; x<3; x++){
			for (int y=0; y<3; y++){
				positions[x][y].setEmpty();		
			};
		};
		player1 = null;
		player2 = null;
		finished = false;
		winner = false;
	}

	private Player getWinner() {
		if (player1.getWinner()) {
			return player1;
		} else {
			if (player2.getWinner()) {
				return player2;
			} else {
				return null;
			}
		}
	}
	
	private Position getPosition(Move move) {
		int x = move.getLine() - 1;
		int y = move.getColumn() - 1;
		return positions[x][y];
	}
	
	private State getState() {
		State state;
		int value = 0;
		Player theWinner = null;
		state = new State();
	// composing the message
		if (! finished) {
			if (player1.getTurn()) {
				state.setStateMessage("Player: " + player1.getName());
			} else {
				state.setStateMessage("Player: " + player2.getName());		
			};			
		} else {
			theWinner = this.getWinner();
			if (theWinner == null) {
				state.setStateMessage("Game ended tied");
			} else {
				state.setStateMessage("Winner: " + theWinner.getName());			
			};			
		};		
	// obtaining board filling	
		for (int x=0; x<3; x++){
			for (int y=0; y<3; y++){
				if (positions[x][y].isOccupied()) {
					value = (positions[x][y].getOccupant()).getSymbol();
					state.setValue((x+1), (y+1), value);			
				} else {
					value = 0;
					state.setValue((x+1), (y+1), value);
				};				
			};
		};
		return state;
	}

	private void evaluateFinished() {
		boolean full = true;
//		boolean result;
		if (winner) {
			finished = true;
		} else {
			for (int x=0; x<3; x++){
				for (int y=0; y<3; y++){
					if (! positions[x][y].isOccupied()) {
						full = false;
					}
				};
			};
			finished = full;
		};
	}

	private void evaluateWinner(Move move) {

		Position p1 = null, p2 = null, selected;
		selected = this.getPosition(move);		
	//test of the column
		switch (move.getLine()) {
			case 1: {
				p1 = positions[1][move.getColumn()-1];
				p2 = positions[2][move.getColumn()-1];
			};
			break;
			case 2:{
				p1 = positions[0][move.getColumn()-1];
				p2 = positions[2][move.getColumn()-1];
			};
			break;
			case 3:{
				p1 = positions[0][move.getColumn()-1];
				p2 = positions[1][move.getColumn()-1];
			};
		};
		winner = selected.isSameOccupant(p1, p2);
		if (winner){
			selected.getOccupant().setWinner(winner);
			finished = true;
		} else {			
	//test of the line
			switch (move.getColumn()) {
				case 1: {
					p1 = positions[move.getLine()-1][1];
					p2 = positions[move.getLine()-1][2];
				};
				break;
				case 2:{
					p1 = positions[move.getLine()-1][0];
					p2 = positions[move.getLine()-1][2];
				};
				break;
				case 3:{
					p1 = positions[move.getLine()-1][0];
					p2 = positions[move.getLine()-1][1];
				};
			};
			winner = selected.isSameOccupant(p1, p2);
			if (winner){
				selected.getOccupant().setWinner(winner);
				finished = true;
			} else {				
	//test of the main diagonal 
				if ((move.getLine() == 1) && (move.getColumn() == 1)) {
					p1 = positions[1][1];
					p2 = positions[2][2];
					winner = selected.isSameOccupant(p1, p2);
				};
				if ((move.getLine() == 2) && (move.getColumn() == 2)) {
					p1 = positions[0][0];
					p2 = positions[2][2];
					winner = selected.isSameOccupant(p1, p2);
				};
				if ((move.getLine() == 3) && (move.getColumn() == 3)) {
					p1 = positions[0][0];
					p2 = positions[1][1];
					winner = selected.isSameOccupant(p1, p2);
				};				
				if (winner){
					selected.getOccupant().setWinner(winner);
					finished = true;
				} else {					
		//test of the secondary diagonal
					if ((move.getLine() == 3) && (move.getColumn() == 1)) {
						p1 = positions[1][1];
						p2 = positions[0][2];
						winner = selected.isSameOccupant(p1, p2);
					};
					if ((move.getLine() == 2) && (move.getColumn() == 2)) {
						p1 = positions[0][2];
						p2 = positions[2][0];
						winner = selected.isSameOccupant(p1, p2);
					};
					if ((move.getLine() == 1) && (move.getColumn() == 3)) {
						p1 = positions[1][1];
						p2 = positions[2][0];
						winner = selected.isSameOccupant(p1, p2);
					};	
					if (winner){
						selected.getOccupant().setWinner(winner);
						finished = true;
					}
				};
			};
		};	
	}
	
}
