package InternalStructure;

import java.util.ArrayList;

import CommonClasses.CommunicationContainer;
import CommonClasses.CommunicationKind;
import CommonClasses.Game;
import CommonClasses.Move;
import CommonClasses.Player;
import CommonClasses.State;
import InterfaceParts.PortAutomatic;
import InterfaceParts.PortAutomaticOutbox;
import InterfaceParts.PortLogic;
import InterfaceParts.PortLogicOutbox;
import InterfaceParts.PortManager2;
import InterfaceParts.PortManagerProxy;
import InterfaceParts.PortManagerProxyOutbox;
import InterfaceParts.PortOpponent;
import InterfaceParts.TTTAutomaticPlayer;
import InterfaceParts.TTTManagerProxy;
import cip.InterfacePort;

public class InternalManager {
		
	protected boolean connected = false;
	protected PortLogic logicPort;
	protected ArrayList<PortManagerProxy> proxyPorts = new ArrayList<PortManagerProxy>();
	protected ArrayList<PortAutomatic> automaticPorts = new ArrayList<PortAutomatic>();
	protected ArrayList<Player> availablePlayers = new ArrayList<Player>();
	protected ArrayList<Player> unavailablePlayers = new ArrayList<Player>();
	protected ArrayList<Game> games = new ArrayList<Game>();
	protected ArrayList<IncompletenessRecord> incompletenessRecords = new ArrayList<IncompletenessRecord>();
	
	public void setConnected() {
		connected = true;
	}
	
	public void setDisconnected() {
		connected = false;
	}
	
	public void setLogicPort(PortLogic aPort) {
		logicPort = aPort;
	}
	
	public void addProxyPort(PortManagerProxy aPort) {
		proxyPorts.add(aPort);
	}
	
	public boolean startServer() {
		PortManagerProxyOutbox anOutbox = (PortManagerProxyOutbox) proxyPorts.get(0).getOutbox();
		boolean result = anOutbox.connect("proxy1");
		return result;		
	}

	public void start(String portId, Player player) {		// start game
		boolean versusAutomatic = player.getAutomatic();
		PortManagerProxy proxy1 = this.getProxyPort(portId);
		Player player2;
		Player player1 = proxy1.getPlayer();
		availablePlayers.remove(player1);
		unavailablePlayers.add(player1);
		if (versusAutomatic) {
			player2 = this.createAutomaticPlayer(player);
		} else {
			String argName = player.getName();
			String player1Name = player1.getName();
			if (argName.equals(player1Name)) {
				player2 = this.getAnyAvailablePlayer();
				if (player2 == null) {
					player2 = this.createAutomaticPlayer(player);
					versusAutomatic = true;
				} else {
					availablePlayers.remove(player2);
					unavailablePlayers.add(player2);
				}
			} else {
				player2 = this.getAvailablePlayer(argName);
				if (player2 == null) player2 = this.getAnyAvailablePlayer();
				if (player2 == null) {
					player2 = this.createAutomaticPlayer(player);
					versusAutomatic = true;
				} else {
					availablePlayers.remove(player2);
					unavailablePlayers.add(player2);
				}
			}
		}
		int player1Symbol = player.getSymbol();
		player1.setSymbol(player1Symbol);
		if (player1Symbol == 1) {
			player2.setSymbol(2);
		} else {
			player2.setSymbol(1);
		}
		boolean player1Starts = player.getTurn();
		if (player1Starts) {
			player1.setTurn(true);
		} else {
			player2.setTurn(true);
		}
		PortLogicOutbox logicOutbox = (PortLogicOutbox) logicPort.getOutbox();
		Game game = logicOutbox.setInitialState(player1, player2);
		game.setPlayer1(player1);
		game.setPlayer2(player2);
		games.add(game);
		State state = game.getGameState();
		if (versusAutomatic) {
			if (! player1Starts) {
				this.automaticMove(player2, portId);
			} else {
				PortManagerProxyOutbox outbox1 = (PortManagerProxyOutbox) proxy1.getOutbox();
				outbox1.updateState(state);	
			}
		} else {
			PortManagerProxy proxy2 = (PortManagerProxy) this.recuperatePlayerPort(player2);
			PortManagerProxyOutbox outbox2 = (PortManagerProxyOutbox) proxy2.getOutbox();
			outbox2.updateState(state);		
			PortManagerProxyOutbox outbox1 = (PortManagerProxyOutbox) proxy1.getOutbox();
			outbox1.updateState(state);	
		}
	}
	
	private void automaticMove(Player automaticPlayer, String portId) {
		PortManagerProxy proxy1 = this.getProxyPort(portId);
		PortAutomatic automaticPort = (PortAutomatic) this.recuperatePlayerPort(automaticPlayer);
		PortAutomaticOutbox automaticOutbox = (PortAutomaticOutbox) automaticPort.getOutbox();
		Game game = this.recuperateGame(automaticPlayer);
		State state = game.getGameState();
		Move move = automaticOutbox.automaticMove(state);
		PortLogicOutbox logicOutbox = (PortLogicOutbox) logicPort.getOutbox();
		Game returnedGame = logicOutbox.treatMove(game, move);
		this.updateGame(game, returnedGame);
		state = game.getGameState();
		PortManagerProxyOutbox outbox1 = (PortManagerProxyOutbox) proxy1.getOutbox();
		outbox1.updateState(state);	
	}

	private InterfacePort recuperatePlayerPort(Player player) {
		if (player.getAutomatic()) {
			for (int i = 0; i<automaticPorts.size(); i++) {
				if (automaticPorts.get(i).getPlayer() == player) return (automaticPorts.get(i));
			}
		} else {
			for (int i = 0; i<proxyPorts.size(); i++) {
				if (proxyPorts.get(i).getPlayer() == player) return (proxyPorts.get(i));
			}
		}
		return null;		
	}

	private Player getAvailablePlayer(String playerName) {
		for (int i = 0; i<availablePlayers.size(); i++) {
			if (availablePlayers.get(i).getName().equals(playerName)) return (availablePlayers.get(i));
		}
		return null;	
	}

	private Player getAnyAvailablePlayer() {
		if (availablePlayers.size() > 0) {
			return (availablePlayers.get(0));
		} else {
			return null;
		}		
	}

	private Player createAutomaticPlayer(Player player) {
		int player1Symbol = player.getSymbol();
		boolean player1Turn = player.getTurn();
		Player player2 = new Player();
		player2.setName("Automatic Player");
		player2.setAutomatic(true);
		if (! player1Turn) player2.setTurn(true);
		if (player1Symbol == 1) {
			player2.setSymbol(2);
		} else {
			player2.setSymbol(1);
		}
		unavailablePlayers.add(player2);
		TTTAutomaticPlayer newComponent = new TTTAutomaticPlayer("TTTAutomaticPlayer");
		newComponent.initialize();
		PortOpponent otherComponentPort = (PortOpponent) newComponent.getPort("port");
		String newPortId = this.getPortAutomaticId();
		PortAutomatic newPort = new PortAutomatic(newPortId);
		newPort.initialize();
		newPort.setPlayer(player2);
		automaticPorts.add(newPort);
		newComponent.connect(newPort, "port");
		newPort.connect(otherComponentPort);
		return player2;
	}
	
	private void updateGame(Game game, Game newGame) {
		game.setGameState(newGame.getGameState());
		game.setGameCode(newGame.getGameCode());
		Player player1 = game.getPlayer1();
		Player player2 = game.getPlayer2();		
		player1.setTurn(newGame.getPlayer1().getTurn());
		player2.setTurn(newGame.getPlayer2().getTurn());
		player1.setWinner(newGame.getPlayer1().getWinner());
		player2.setWinner(newGame.getPlayer2().getWinner());			
	}

	public void sendMessage(String portId, String message) {
		PortManagerProxy proxy1 = this.getProxyPort(portId);
		Player player1 = proxy1.getPlayer();
		Game game = this.recuperateGame(player1);
		Player player2 = game.getOpponent(player1);
		boolean versusAutomatic = player2.getAutomatic();
		if (! versusAutomatic) {
			PortManagerProxy proxy2 = (PortManagerProxy) this.recuperatePlayerPort(player2);
			PortManagerProxyOutbox outbox2 = (PortManagerProxyOutbox) proxy2.getOutbox();
			outbox2.receiveMessage(message);
		}
	}

	public void makeMove(String portId, Move move) {
		PortManagerProxy proxy1 = this.getProxyPort(portId);
		Player player1 = proxy1.getPlayer();
		Game game = this.recuperateGame(player1);
		Player player2 = game.getOpponent(player1);
		PortLogicOutbox logicOutbox = (PortLogicOutbox) logicPort.getOutbox();
		Game newGame = logicOutbox.treatMove(game, move);
		int returnedCode = newGame.getGameCode();
		if (returnedCode ==3) {
			CommunicationContainer result = new CommunicationContainer();
			result.setKind(CommunicationKind.connectedPlayerNotification);
			result.setContent("Irregular move");
			((PortManagerProxyOutbox) proxy1.getOutbox()).communicate(result);
		} else {
			this.updateGame(game, newGame);
			State state = game.getGameState();
			((PortManagerProxyOutbox) proxy1.getOutbox()).updateState(state);
			boolean versusAutomatic = player2.getAutomatic();
			if (versusAutomatic) {
				if (returnedCode == 4) {
					this.automaticMove(player2, portId);
					game = this.recuperateGame(player1);
					returnedCode = game.getGameCode();
				}
			} else {
				PortManagerProxy proxy2 = (PortManagerProxy) this.recuperatePlayerPort(player2);
				PortManagerProxyOutbox outbox2 = (PortManagerProxyOutbox) proxy2.getOutbox();
				outbox2.updateState(state);				
			}
			if (returnedCode == 5 || returnedCode == 6) {
				if (versusAutomatic) {
					this.discardAutomaticPlayer(player2);
				} else {
					this.makeAvailable(player2);
				}
				this.makeAvailable(player1);
				this.discardGame(game);
			}
		}
		
	}

	private void discardGame(Game game) {
		game.setPlayer1(null);
		game.setPlayer2(null);
		games.remove(game);		
	}

	private void makeAvailable(Player player) {
		player.setSymbol(0);
		player.setTurn(false);
		player.setWinner(false);
		unavailablePlayers.remove(player);
		availablePlayers.add(player);		
	}

	private void discardAutomaticPlayer(Player player) {
		PortAutomatic automaticPort = (PortAutomatic) this.recuperatePlayerPort(player);
		PortAutomaticOutbox automaticOutbox = (PortAutomaticOutbox) automaticPort.getOutbox();
		automaticOutbox.selfDestruct();
		automaticPorts.remove(automaticPort);
		unavailablePlayers.remove(player);
	}

	private Game recuperateGame(Player player) {
		for (int i = 0; i<games.size(); i++) {
			if ((games.get(i).getPlayer1() == player) || (games.get(i).getPlayer2() == player)) {
				return (games.get(i));
			}		
		}
		return null;
	}
	
	public void giveUp(String portId, String playerId) {
		PortManagerProxy proxy1 = this.getProxyPort(portId);
		Player player1 = proxy1.getPlayer();
		Game game = this.recuperateGame(player1);
		PortLogicOutbox logicOutbox = (PortLogicOutbox) logicPort.getOutbox();
		Game newGame = logicOutbox.finishWithdrawal(game, player1);
		this.updateGame(game, newGame);
		State state = game.getGameState();
		PortManagerProxyOutbox outbox1 = (PortManagerProxyOutbox) proxy1.getOutbox();
		outbox1.updateState(state);		
		Player player2 = game.getOpponent(player1);
		boolean versusAutomatic = player2.getAutomatic();
		if (versusAutomatic) {
			this.discardAutomaticPlayer(player2);
		} else {
			PortManagerProxy proxy2 = (PortManagerProxy) this.recuperatePlayerPort(player2);
			PortManagerProxyOutbox outbox2 = (PortManagerProxyOutbox) proxy2.getOutbox();
			outbox2.updateState(state);
			this.makeAvailable(player2);
		}
		this.makeAvailable(player1);
		this.discardGame(game);		
	}

	public void getConnectedPlayers(String portId) {
		CommunicationContainer result = new CommunicationContainer();
		result.setKind(CommunicationKind.connectedPlayerNotification);
		if (availablePlayers.size() > 0) {
			String content = "";
			for (int i=0; i<availablePlayers.size(); i++) {
				Player aPlayer = availablePlayers.get(i);
				String name = aPlayer.getName();
				content = content + name + ", ";				
			}
			result.setContent(content);
			PortManagerProxy aPort = this.getProxyPort(portId);
			((PortManagerProxyOutbox) aPort.getOutbox()).communicate(result);
		}
	}

	public void disconnect(String portId, String arg0) {
		PortManagerProxy port = this.getProxyPort(portId);
		CommunicationContainer notification = new CommunicationContainer();
		notification.setKind(CommunicationKind.disconnectionNotification);
		notification.setContent("Disconnected from TTT Server");
		PortManagerProxyOutbox outbox = (PortManagerProxyOutbox) port.getOutbox();
		outbox.communicate(notification);
		Player player = port.getPlayer();
		port.setPlayer(null);
		availablePlayers.remove(player);
		proxyPorts.remove(port);
//		PortManager2 externalPort = (PortManager2) outbox.getExternalPort();
//		externalPort.disconnect();
		outbox.selfDestruct();
		port.disconnect();
	}
	
	public void connect(String portId, String arg0) {
	//	register a new player	
		Player player = new Player();
		String playerId;
		if (arg0.isEmpty()) {
			playerId = "player_" + portId;
		}
		else {
			playerId = arg0;
		}
		player.setName(playerId);
		player.setAutomatic(false);
		availablePlayers.add(player);
		CommunicationContainer notification = new CommunicationContainer();
		notification.setKind(CommunicationKind.connectionNotification);
		notification.setContent("Connected to TTT Server");
		PortManagerProxy firstProxy = this.getProxyPort(portId);
		firstProxy.setPlayer(player);
		PortManagerProxyOutbox firstOutbox = (PortManagerProxyOutbox) firstProxy.getOutbox();
		firstOutbox.communicate(notification);
	// new 	component TTTManagerProxy
		TTTManagerProxy newTTTManagerProxy = new TTTManagerProxy("TTTManagerProxy");
		newTTTManagerProxy.initialize();
		PortManager2 otherComponentPort = (PortManager2) newTTTManagerProxy.getPort("manager");
	// conect to new component to wait for the next player	
		String newPortName = this.getPortManagerId();
		PortManagerProxy newProxy = new PortManagerProxy(newPortName);
		newProxy.initialize();
		newProxy.setInternalReference(this);
		proxyPorts.add(newProxy);
		PortManagerProxyOutbox newOutbox = (PortManagerProxyOutbox) newProxy.getOutbox();
		newTTTManagerProxy.connect(newProxy, "manager");
		newProxy.connect(otherComponentPort);
		newOutbox.connect(newPortName);
	}
	
	private String getPortManagerId() {
		int number = proxyPorts.size() + 1;
		int portNumber = this.evaluatePortManagerId(number);
		return ("proxy" + String.valueOf(portNumber));
	}
	
	private int evaluatePortManagerId(int number) {
		PortManagerProxy existentPort = this.getProxyPort("proxy" + String.valueOf(number));
		if (existentPort == null) {
			return number;
		} else {
			return this.evaluatePortManagerId(number +1);
		}		
	}

	private String getPortAutomaticId() {
		int number = automaticPorts.size() + 1;
		int portNumber = this.evaluatePortAutomaticId(number);
		return ("automatic" + String.valueOf(portNumber));
	}

	private int evaluatePortAutomaticId(int number) {
		PortAutomatic existentPort = this.getAutomaticPort("automatic" + String.valueOf(number));
		if (existentPort == null) {
			return number;
		} else {
			return this.evaluatePortAutomaticId(number +1);
		}		
	}

	private PortManagerProxy getProxyPort(String portId) {
		for (int i=0; i<proxyPorts.size(); i++) {
			PortManagerProxy aPort = proxyPorts.get(i);
			if (aPort.isYourId(portId)) return aPort;
		}
		return null;
	}
	
	private PortAutomatic getAutomaticPort(String portId) {
		for (int i=0; i<automaticPorts.size(); i++) {
			PortAutomatic aPort = automaticPorts.get(i);
			if (aPort.isYourId(portId)) return aPort;
		}
		return null;
	}

}
